local L = YA_locale()
local addonName = "YorsahjAnnounce"
local LaddonName = L[addonName]
YorsahjAnnounce = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceTimer-3.0")
local addon = YorsahjAnnounce

local defaults = {
  profile = {
    debug = false, -- for addon debugging
    announcekill = true,
    pingkill = true, 
    announcestrat = true,
    prestrat = true,
    spamcount = 1,
    minimap = {
        hide = false,
    },
    lfr = {
      usepriority = true,
      priority = {
        "purple",
	"yellow",
	"blue",
	"black",
	"red",
	"green",
      },
      manual = {}
    },
    normal = {
      usepriority = true,
      priority = {
        "purple",
	"green",
	"blue",
	"yellow",
	"black",
	"red",
      },
      manual = {}
    },
    heroic = {
      usepriority = false,
      priority = {
	"green",
	"yellow",
        "purple",
	"blue",
	"black",
	"red",
      },
      manual = {
	[105437] = "yellow",
	[105440] = "purple",
      }
    },
  }
}
local colordata = {
  ["purple"] = {
    name = L["Purple"],
    emph = "{rt3}",
    code = "\124cffff00ff",
    desc = L["Healing debuff"],
    strat = L["Healing debuff"],
    position = 8,
    order = 1,
  },
  ["blue"] = {
    name = L["Blue"],
    emph = "{rt6}",
    code = "\124cff0000ff",
    desc = L["Mana void add drains mana"],
    strat = L["Kill mana void add"],
    position = 7,
    order = 4,
  },
  ["green"] = {
    name = L["Green"],
    emph = "{rt4}",
    code = "\124cff00ff00",
    desc = L["Splash damage (except in LFR)"],
    strat = L["SPREAD OUT"],
    position = 2,
    order = 2,
  },
  ["black"] = {
    name = L["Black"],
    emph = "{rt8}",
    code = "\124cff000000",
    desc = L["Many adds"],
    strat = L["AOE adds"],
    position = 6,
    order = 5,
  },
  ["red"] = {
    name = L["Red"],
    emph = "{rt7}",
    code = "\124cffff0000",
    desc = L["Chain fire damage"],
    strat = L["STACK"],
    position = 1,
    order = 6,
  },
  ["yellow"] = {
    name = L["Yellow"],
    emph = "{rt1}",
    code = "\124cffffff00",
    desc = L["Soft enrage"],
    strat = L["Soft enrage"],
    position = 4,
    order = 3,
  },
}

local spelldata = {
   [105420] = { "purple", "green", "blue", heroic = "black" },
   [105435] = { "green", "red", "black", heroic = "blue" },
   [105436] = { "green", "yellow", "red", heroic = "black"},   
   [105437] = { "purple", "blue", "yellow", heroic = "green" },   
   [105439] = { "blue", "black", "yellow", heroic = "purple" },   
   [105440] = { "purple", "red", "black", heroic = "yellow" },   
}

local buffdata = {
   [104894] = "black",
   [104896] = "purple",
   [104897] = "red",
   [104898] = "green",
   [104901] = "yellow",
   [105027] = "blue",
}

-- canonical orders used for output
local colorcanon = {}
for c,e in pairs(colordata) do
  colorcanon[e.order] = c
end
local spellcanon = {}
for id,_ in pairs(spelldata) do
  table.insert(spellcanon, id)
end
table.sort(spellcanon)

local numcolors = 6
local settings = defaults.profile
local optionsFrame
local charName
local hiddenFrame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
local revision = tonumber(("$Revision: 32 $"):match("%d+"))
local minimapIcon = LibStub("LibDBIcon-1.0")
local aggroStr = ("Iilth qi'uothk shn'ma yeh'glu Shath'Yar! H'IWN IILTH"):gsub("%p"," "):upper()
local LDB, LDBo

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end
local function debug(msg)
  if addon.db.profile.debug then
     chatMsg(msg)
  end
end

-- append two strings with a separator if the first is nonempty
local function strappend(s1,s2,sep)
  s1 = s1 or ""
  s2 = s2 or ""
  if #s1 > 0 then s1 = s1..sep end
  return s1..s2
end

function addon:myOptions() 
return {
  type = "group",
  set = function(info,val)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          s[info[#info]] = val; debug(info[#info].." set to: "..tostring(val))
          addon:Update()
        end,
  get = function(info)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          return s[info[#info]] end,
  args = {
   general = {
    type = "group",
    inline = true,
    name = L["General"],
    args = {
      debug = {
        name = L["Debug"],
        desc = L["Toggle debugging output"],
        type = "toggle",
        guiHidden = true,
      },
      config = {
        name = L["Config"],
        desc = L["Open the configuration GUI"],
        type = "execute",
        guiHidden = true,
        func = function() addon:Config() end,
      },
      announcekill = {
        order = 10,
        name = L["Announce Target"],
        desc = L["Announce Ooze to kill (requires raid lead or assist)"],
        type = "toggle",
      },
      spamcount = {
        order = 10.5,
	name = L["Announce repeat"],
        desc = L["How many times to repeat the kill announcement raid warning"],
	disabled = function() return not settings.announcekill end,
	type = "range",
	min = 1,
	max = 5,
	step = 1,
      },
      pingkill = {
        order = 11,
        name = L["Ping Minimap"],
        desc = L["Ping the minimap for the ooze kill location"],
        type = "toggle",
	disabled = function() return not settings.announcekill end,
      },
      announcestrat = {
        order = 12,
        name = L["Announce Strat"],
        desc = L["Announce strategy once oozes arrive (requires raid lead or assist)"],
        type = "toggle",
      },
      prestrat = {
        order = 13,
	--disabled = function() return not settings.announcestrat end,
        name = L["Pre-announce Strat"],
        desc = L["Pre-announce strategy during ooze killing"],
        type = "toggle",
	disabled = function() return not settings.announcekill end,
      },
      minimap = {
        order = 5,
	width = "double",
        name = L["Minimap Icon"],
        desc = L["Display minimap icon"],
        type = "toggle",
        set = function(info,val)
          settings.minimap.hide = not val
          addon:Update()
	end,
        get = function() return not settings.minimap.hide end,
      },
      reset = {
        order = 6,
        type = "execute",
	confirm = true,
	confirmText = L["Are you certain you want to reset all settings for this addon?"],
        name = L["Reset to defaults"],
        func = function() addon:ResetSettings() end,
      },
      lfr = {
        order = 100,
	name = L["Looking for Raid"],
	type = "group",
	args = addon:diffconfig("lfr"),
      },
      normal = {
        order = 120,
	name = L["Normal"],
	type = "group",
	args = addon:diffconfig("normal"),
      },
      heroic = {
        order = 140,
	name = L["Heroic"],
	type = "group",
	args = addon:diffconfig("heroic"),
      },
      }
     }
    }
  }
end

function addon:manualSetting(diff, spellid)
  local ml = settings[diff].manual
  local colors = spelldata[spellid]
  if ml[spellid] then return ml[spellid]
  else
    for _,c in ipairs(settings[diff].priority) do
      for k,color in pairs(colors) do
        if k ~= "heroic" or k == diff then
           if color == c then return c end
        end
      end
    end
  end
end

function addon:diffconfig(diff)      
  local ret = {
      priority = {
	    order = 10,
	    type = "group",
	    inline = true,
	    name = L["Color Priority"],
	    args = {
	      enable = {
	        order = 1,
	        type = "toggle",
		name = L["Use Color Priority"],
		set = function(info, val) 
		   debug("priority for "..diff.." = "..(val and "on" or "off"))
		   settings[diff].usepriority = val
		end,
		get = function() return settings[diff].usepriority end,
	      },
              announce = {
                order = 2,
                type = "execute",
                name = L["Announce priority"],
                disabled = function() return not settings[diff].usepriority end,
                func = function() 
                  local str = L["Ooze color priority is: "]
                  local s = ""
                  for _,c in ipairs(settings[diff].priority) do
		    s = strappend(s,colordata[c].name," > ")
                  end
		  local chan = UnitInRaid("player") and "RAID" or "SAY"
                  SendChatMessage(str..s, chan)
                end,
              },
	    }
      },
      manual = {
	    order = 20,
	    type = "group",
	    inline = true,
	    name = L["Manual Selection"],
	    args = {
	      enable = {
	        order = 1,
	        type = "toggle",
		name = L["Use Manual Selection"],
		set = function(info, val) 
		   debug("priority for "..diff.." = "..(val and "off" or "on"))
		   settings[diff].usepriority = not val
		end,
		get = function() return not settings[diff].usepriority end,
	      },
              announce = {
                order = 2,
                type = "execute",
                name = L["Announce list"],
                disabled = function() return settings[diff].usepriority end,
                func = function() 
                  for _,spellid in ipairs(spellcanon) do
		    local colors = spelldata[spellid]
                    local s = ""
                    local killcolor = addon:manualSetting(diff,spellid)
		    for _,color in ipairs(colorcanon) do
                      for k,c in pairs(colors) do
                        if color == c and (k ~= "heroic" or k == diff) then
                          local n = colordata[color].name
                          if color == killcolor then
                            n = ">>"..n:upper().."<<"
                          else
                            n = n:lower()
                          end
			  s = strappend(s, n, "  ")
			end
                      end
                    end
		    local chan = UnitInRaid("player") and "RAID" or "SAY"
                    SendChatMessage(s, chan)
                  end
                end,
              },
	    }
      }
    }
  local pt = ret.priority.args
  for i = 1, numcolors do
    pt["order"..i] = {
      order = 10*i,
      type = "description",
      fontSize = "large",
      disabled = function() return not settings[diff].usepriority end,
      name = function() 
        local pl = settings[diff].priority
        local e = colordata[pl[i]]
	local tc = settings[diff].usepriority and e.code or "\124cff808080"
	return tc .. e.name .. "\124r - " .. e.desc
      end,
    }
    pt["orderup"..i] = {
        type = "execute",
	order = 10*i + 2,
	name = L["Up"],
	width = "half",
	disabled = function() return (i == 1) or not settings[diff].usepriority end,
	func = function()
          local pl = settings[diff].priority
	  local temp = pl[i-1]
	  pl[i-1] = pl[i]
	  pl[i] = temp
	end,
    }
    pt["orderdown"..i] = {
        type = "execute",
	order = 10*i + 4,
	name = L["Down"],
	width = "half",
	disabled = function() return (i == numcolors) or not settings[diff].usepriority end,
	func = function()
          local pl = settings[diff].priority
	  local temp = pl[i+1]
	  pl[i+1] = pl[i]
	  pl[i] = temp
	end,
    }
  end
  local mt = ret.manual.args
  for spellid, colors in pairs(spelldata) do
    ret.manual.args["spell"..spellid] = {
      name = "spell:"..spellid,
      order = spellid,
      type = "select",
      style = "radio",
      width = "half",
      disabled = function() return settings[diff].usepriority end,
      values = function() 
        local ret = {}
        for k,color in pairs(colors) do
           if k ~= "heroic" or k == diff then
	      local e = colordata[color]
	      local tc = settings[diff].usepriority and "\124cff808080" or e.code
              ret[e.order] = tc..e.name.."\124r"
           end
        end
        return ret
      end,
      set = function(info, val) settings[diff].manual[spellid] = colorcanon[val] end,
      get = function(info) return colordata[addon:manualSetting(diff,spellid)].order end
    }
  end
  return ret
end

local function table_clone(t)
  if not t then return nil
  elseif type(t) == "table" then
    local res = {}
    for k,v in pairs(t) do
      res[table_clone(k)] = table_clone(v)
    end
    return res
  else
    return t
  end
end

function addon:RefreshConfig()
  -- things to do after load or settings are reset
  debug("RefreshConfig")
  settings = addon.db.profile
  addon.settings = settings
  charName = UnitName("player")
  for k,v in pairs(defaults.profile) do
     if settings[k] == nil then
       settings[k] = table_clone(v)
     end
  end
  local check = {}
  for _,diff in pairs({"lfr","normal","heroic"}) do -- consistency check priorities
    local ok = #settings[diff].priority == numcolors
    if ok then
      wipe(check)
      for i,c in pairs(settings[diff].priority) do
        if not tonumber(i) or i < 1 or i > numcolors or 
	   not colordata[c] or check[c] then
	  ok = false
	  break 
	end
	check[c] = true
      end
    end
    if ok then
      for spellid,c in pairs(settings[diff].manual) do
        local d = spelldata[spellid]
	if not d then ok = false; break end
        local gotit = false
	for i,cc in pairs(d) do
	  if c == cc and (i == diff or tonumber(i)) then
	    gotit = true
	  end
	end
	if not gotit then ok = false; break end
      end
    end
    if not ok then
      settings[diff] = table_clone(defaults.profile[diff])
      chatMsg("Consistency check failed - Resetting "..diff.." color settings")
    end
  end
  settings.loaded = true
  addon:Update()
end

function addon:ResetSettings()
   local shown = InterfaceOptionsFrame:IsShown() 
   if shown then
     addon:Config()
   end
   addon:RefreshConfig()
   for k,v in pairs(defaults.profile) do settings[k] = table_clone(v) end
   if shown then
     addon:Config()
   end
end

function addon:Update()
  -- things to do when settings change
  if LDBo then
    if settings.minimap.hide then
      minimapIcon:Hide(addonName)
    else
      minimapIcon:Show(addonName)
    end
  end
  addon:CheckEnable()
end


function addon:OnInitialize()
  SLASH_YORSAHJANNOUNCE1 = nil
  SlashCmdList["YORSAHJANNOUNCE"] = nil
  hash_SlashCmdList["/yorsahjannounce"] = nil
  SLASH_RTC1 = nil
  SlashCmdList["YA"] = nil
  hash_SlashCmdList["/ya"] = nil

  addon.db = LibStub("AceDB-3.0"):New("YorsahjAnnounceDB", defaults)
  addon:RefreshConfig()
  local options = addon:myOptions()
  LibStub("AceEvent-3.0"):Embed(addon)
  LibStub("AceConfigRegistry-3.0"):ValidateOptionsTable(options, addonName)
  LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options, {"yorsahjannounce", "ya"})
  optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, addonName, nil, "general")
  optionsFrame.default = function() addon:ResetSettings() end
  options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, L["Profiles"], addonName, "profiles")

  debug("OnInitialize")

  self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
  self.db.RegisterCallback(self, "OnDatabaseReset", "RefreshConfig")
  addon.init = true
end

function addon:Config()
  if optionsFrame then
    if ( InterfaceOptionsFrame:IsShown() ) then
      InterfaceOptionsFrame:Hide();
    else
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
      local f = InterfaceOptionsFrameAddOnsListScrollBar 
      if f and f.GetMinMaxValues and f.SetValue then
        local min,max = f:GetMinMaxValues()
        f:SetValue(max)
      end
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
    end
  end
end


function addon:OnEnable()
  debug("OnEnable")
  self:RegisterEvent("ZONE_CHANGED","CheckEnable")
  self:RegisterEvent("ZONE_CHANGED_INDOORS","CheckEnable")
  self:RegisterEvent("ZONE_CHANGED_NEW_AREA","CheckEnable")
  self:RegisterEvent("UNIT_EXITED_VEHICLE","CheckEnable")
  self:RegisterEvent("RAID_ROSTER_UPDATE","CheckEnable")
  self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
  self:RegisterEvent("CHAT_MSG_ADDON")
  RegisterAddonMessagePrefix(addonName)
 
  if LDB then
    return
  end
  if AceLibrary and AceLibrary:HasInstance("LibDataBroker-1.1") then
    LDB = AceLibrary("LibDataBroker-1.1")
  elseif LibStub then
    LDB = LibStub:GetLibrary("LibDataBroker-1.1",true)
  end
  if LDB then
    LDBo = LDB:NewDataObject(addonName, {
        type = "launcher",
        label = addonName,
        icon = "Interface\\Icons\\achievement_doublerainbow",
        OnClick = function(self, button)
                if button == "RightButton" then
                        addon:Config()
                else
                        addon:CheckEnable()
                end
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine then
                        tooltip:SetText(addonName)
                        --tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to run a check"])
                        tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["for options"])
                        tooltip:Show()
                end
        end,
     })
  end 

  if LDBo then
    minimapIcon:Register(addonName, LDBo, settings.minimap)
  end
  addon:Update()
end

function addon:GetDiff()
  local instname, insttype, diff, diffname, maxPlayers, playerDifficulty, isDynamicInstance = GetInstanceInfo()
  if HasLFGRestrictions() or GetLFGMode() == "lfgparty" then 
     return "lfr"
  elseif diff == 3 or diff == 4 then
     return "heroic"
  else
     return "normal"
  end
end

function addon:InShuma()
  local oldmap = GetCurrentMapAreaID()
  local oldlvl = GetCurrentMapDungeonLevel()
  SetMapToCurrentZone()
  local map = GetCurrentMapAreaID()
  local lvl = GetCurrentMapDungeonLevel()
  local shuma = (map == 824 and lvl == 3)
  local unitX, unitY = GetPlayerMapPosition("player")
  if not unitX then unitX,unitY = 0,0 end
  unitX, unitY = unitX*100, unitY*100
  SetMapByID(oldmap)
  if lvl and lvl > 0 then
    SetDungeonMapLevel(oldlvl)
  end
  return shuma, unitX, unitY
end

function addon:CheckEnable()
  if not addon.init then return end
  --debug("CheckEnable")
  if addon:InShuma() and not IsRaidOfficer() and not addon.offwarn then
    chatMsg(L["You must have raid lead or assist to make annoucements."])
    addon.offwarn = true
  end
  if addon:InShuma() and (IsRaidOfficer() or addon.db.profile.debug) then
    if addon.enabled then return end
    addon.enabled = true
    addon.offwarn = false
    chatMsg(L["Addon enabled. Type '/ya config' for options."])
    addon:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  else -- wrong zone
    if not addon.enabled then return end
    chatMsg(L["Addon disabled."])
    addon.enabled = false
    addon:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    addon:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  end

end

function addon:UNIT_SPELLCAST_SUCCEEDED(event, unitid, spellName, _, _, spellID)
  local call = spelldata[spellID]
  if call then -- boss called oozes
    debug("Boss called "..spellID)
    local now = GetTime()
    addon.lastcall = addon.lastcall or 0
    if now < addon.lastcall + 10 then
      return -- ignore duplicate events
    end
    addon.lastcall = now
    if addon.colorhits then wipe(addon.colorhits) end
    local diff = addon:GetDiff()
    local colors = {}
    for k,v in pairs(call) do
      if k ~= "heroic" or k == diff then
        colors[v] = true
      end
    end
    local killcolor
    if settings[diff].usepriority then -- color priority
      for _,c in ipairs(settings[diff].priority) do
        if colors[c] then
	  killcolor = c
	  break
	end
      end
    else -- manual setting
      killcolor = addon:manualSetting(diff,spellID)
    end
    local msg = L["Kill"].." "..colordata[killcolor].name
    local emph = colordata[killcolor].emph
    debug(msg)
    emph = emph..emph..emph
    msg = emph.." "..msg:upper().."!!! "..emph
    colors[killcolor] = false
    local strat = addon:strat(colors)
    debug(strat)
    --addon:PingColor(killcolor) -- debugging only
    if IsRaidOfficer() and settings.announcekill and not addon.suppress then 
       for i = 1,settings.spamcount do
         SendChatMessage(msg, "RAID_WARNING") 
       end
       if settings.pingkill then
         addon:PingColor(killcolor)
       end
       if settings.prestrat then
         SendChatMessage(L["Prepare for: "]..strat,"RAID")
       end
    end
  end
end

function addon:strat(colorhits)
      local str = ""
      local name = ""
      if colorhits["green"] and addon:GetDiff() ~= "lfr" then
        str = colordata["green"].strat -- spread out
      else
        str = colordata["red"].strat -- stack
      end
      for _,c in ipairs(colorcanon) do
        if colorhits[c] then
          name = strappend(name,colordata[c].name,"/")
          if c ~= "red" and c ~= "green" then
	    str = strappend(str, colordata[c].strat, ", ")
	  end
	end
      end
      return name..": "..str
end

function addon.ReapColors() 
  local now = GetTime()
  addon.laststrat = addon.laststrat or 0
  if now < addon.laststrat + 10 then -- only post it once
    return
  end
  local numhits = 0
  for _,_ in pairs(addon.colorhits) do numhits = numhits+1 end
  if numhits < 2 or (numhits < 3 and addon:GetDiff() == "heroic") then return end
  if addon.db.profile.announcestrat then
      local str = addon:strat(addon.colorhits)
      debug(str)
      if IsRaidOfficer() and not addon.suppress then 
         SendChatMessage(str, "RAID_WARNING") 
      end
  end
  addon.laststrat = now
  wipe(addon.colorhits)
end

function addon:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
  local subevent = select(2, ...)
  if subevent == "SPELL_AURA_APPLIED" then
    local spellID = select(12,...)
    local buff = buffdata[spellID]
    if buff then -- boss gained an ooze
      local now = GetTime()
      addon.lastcall = addon.lastcall or 0
      if now < addon.lastcall + 8 then
        return -- ignore buffs from call
      end
      debug("Boss gained: "..buff.." : "..spellID)
      addon.colorhits = addon.colorhits or {}
      addon.colorhits[buff] = true
      addon:ScheduleTimer(addon.ReapColors,1)
    end
  end
end

-- handshake to ensure only one copy of the addon is announcing
function addon:CHAT_MSG_MONSTER_YELL(event, message, ...)
  if message:gsub("%p"," "):upper():match(aggroStr) then
   debug("Detected boss pull")
   if addon.enabled and settings.announcekill and IsRaidOfficer() then
     addon.suppress = false
     SendAddonMessage(addonName, "ENGAGE ["..revision.."]", "RAID")
     addon:ScheduleTimer(function()
       if not addon.suppress then
         SendChatMessage("<"..addonName.."> "..L["Boss engaged: Addon activated. Good luck!"], "RAID")
       end
     end, 2)
   end
  end
end
function addon:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
  if prefix ~= addonName then return end
  debug("CHAT_MSG_ADDON: "..prefix..":"..message..":"..channel..":"..sender)
  if UnitIsUnit(sender, "player") then return end
  local srev = message:match("^ENGAGE %[(%d+)%]")
  srev = tonumber(srev or 0) or 0
  debug("Got engage message v"..srev)
  if UnitIsPartyLeader(sender) or (not UnitIsPartyLeader("player") and 
     (srev > revision or (srev == revision and sender < UnitName("player")))) then -- prefer the leader or newest version
    chatMsg(L["Addon temporarily disabled in favor of"].." "..sender)
    addon.suppress = true
  end
end

function addon:PingColor(color)
  local _, unitX, unitY = addon:InShuma()
  local centerX, centerY = 47, 47
  local distsq = math.pow(unitX - centerX,2) + math.pow(unitY - centerY,2)
  if distsq < 100 then -- standing close to center
    local radius = Minimap:GetHeight()*Minimap:GetEffectiveScale()/2
    local theta = (colordata[color].position * (math.pi/4)) - (math.pi/8)
    local rotate = GetCVar("rotateMinimap")
    if rotate and tonumber(rotate) ~= 0 then
      theta = theta - GetPlayerFacing()
    end
    local pingX = -1 * radius * math.sin(theta)
    local pingY = radius * math.cos(theta)
    Minimap:SetZoom(0)
    Minimap:PingLocation(pingX, pingY)
  end
  SendAddonMessage("DBM-YORSAHJARROW",color,"RAID") -- DBM arrow integration
end
